// Author: Vladimir Savic
// E-mail: vladimirsvc@yahoo.com

#include "LeftRotationState.h"
#include "RightRotationState.h"
#include "LeftAngleState.h"
#include "RightAngleState.h"
#include "IdleState.h"
#include "TimerState.h"
#include "SpeedState.h"
#include "StateMachine.h"
#include "TransitionMatrix.h"

// transition matrix:
static BYTE* ptrCurrentState    ;               // pointer to current state
static BYTE  currentStateIndex  ;               // index of currently executed state
static BYTE  prevStateIndex     ;               // index of previous executed state

static TransitionMatrix* ptrTransitionMatrix;   // pointer to transition matrix
static StateMachineAbstract* ptr;

extern BYTE command;

// ********************************************************************
// *  states encoding                                                 *
// *  S1 -> 0x00 (first  transition matrix row, index 0) - states[0]  *
// *  S2 -> 0x01 (second transition matrix row, index 1) - states[1]  *
// *  S3 -> 0x02 (third  transition matrix row, index 2) - states[2]  *
// *  S4 -> 0x03 (fourth transition matrix row, index 3) - states[3]  *
// ********************************************************************
static StateAbstract states[NUMBER_OF_STATES];

void Initialization( void* ptrSma ){
    ptr = (StateMachineAbstract*)ptrSma;
    
    ptr->IsStarted = 0x00;
    
    // State machine methods initialization:
    ptr->StateInitialization            = &StateInitialization            ;
    ptr->TransitionMatrixInitialization = &TransitionMatrixInitialization ;
    ptr->ProcessCommand                 = &ProcessCommand                 ;
    ptr->StartProcessing                = &StartProcessing                ;
}

static void StateInitialization( void ){
    states[0].StateInitialization = &IdleStateInitialization          ; // Idle state, start state
    states[1].StateInitialization = &LeftRotationStateInitialization  ; // Left rotation state
    states[2].StateInitialization = &RightRotationStateInitialization ; // Right rotation state
    states[3].StateInitialization = &TimerStateInitialization         ; // Timer state

    states[0].StateInitialization(&states[0]);
    states[1].StateInitialization(&states[1]);
    states[2].StateInitialization(&states[2]);
    states[3].StateInitialization(&states[3]);
}

void TransitionMatrixInitialization( void* tm ){
    ptrTransitionMatrix = (TransitionMatrix*)tm;
}

void StartProcessing( void ){
    command  = 0xFF;
    ptrCurrentState  = 0;
    DoTransition(ptrTransitionMatrix->startState);
    ptr->IsStarted = 0xFF;
}

void ProcessCommand ( void ){
    BYTE newState = 0xFF;

    // transition matrix method GetNextState returns new state code
    newState = ptrTransitionMatrix->GetNextState( currentStateIndex, prevStateIndex, states[currentStateIndex].Flags, command );
    
    if( newState <= NUMBER_OF_STATES - 1 ) DoTransition ( &newState );

    command = 0xFF;
}

static void DoTransition(void* state){
    // ptrCurrentState  - pointer to DT byte
    // ptrPreviousState - pointer to PS byte
    // ------------------------------------------------------------------------------
    // |    DT    |     PS   |    TPS   |   Cmd1   |   Cmd2   |   Cmd3   |   Cmd4   |
    // ------------------------------------------------------------------------------
    // |<- BYTE ->|<- BYTE ->|<- BYTE ->|<- BYTE ->|<- BYTE ->|<- BYTE ->|<- BYTE ->|
    prevStateIndex    = currentStateIndex;
    currentStateIndex = *(BYTE*)state;
    ptrCurrentState   = ptrTransitionMatrix->buffer + (ptrTransitionMatrix->xModeBytes * currentStateIndex);   // calculate current state pointer
    states[currentStateIndex].DefaultStateMethod(&command);                                                    // execute default state method
}