// Author: Vladimir Savic
// E-mail: vladimirsvc@yahoo.com

#include "DataTypes.h"
#include "StateAbstract.h"

#ifndef _TRANSITION_MATRIX_
#define _TRANSITION_MATRIX_

//     <-                                  NUMBER_OF_TM_COLUMNS                                  ->
//     <-                   3               -><-           external commands 4                   ->
//     --------------------------------------------------------------------------------------------
//  S1 |<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|
//     --------------------------------------------------------------------------------------------
//  S2 |<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|
//     --------------------------------------------------------------------------------------------
//  ...|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|<- xxMode ->|
//     --------------------------------------------------------------------------------------------

#define NUMBER_OF_STATES   4
#define NUMBER_OF_EXT_CMD  4
#define X_MODE_BYTES       4
#define X_MODE_NOTHING     0x0F

// **********************************************************************
// * States encoding modes:                                             *
// * x1 mode (0x00) - 8 bits, max number of state  256 - 1 0xFF nothing *
// * x2 mode (0x01) - 4 bits, max number of states  16 - 1 0x0F nothing *
// **********************************************************************

// ****************************
// * Priority encoding modes: *
// * DT  mode (0x00)          *
// * PS  mode (0x01)          *
// * Cmd mode (0x02)          *
// ****************************

typedef struct {
    // properties:
    BYTE* buffer     ;
    BYTE* startState ;
    BYTE  xModeBytes ;
    BYTE  xMode      ;
    
    // methods:
    void ( *Initialization ) ( void* );
    BYTE ( *GetNextState   ) ( BYTE stateIndex, BYTE prevStateIndex, BYTE stateFlag, BYTE cmd ); // first param: current state, previous second param: command
} TransitionMatrix;

// Transition matrix prototypes methods:
       void PriorityMethodsInitialization ( void* );
static BYTE GetNextState  ( BYTE stateIndex, BYTE prevStateIndex, BYTE stateFlag, BYTE cmd );
static BYTE CalcNextState ( BYTE prioritCode );

/*static BYTE X1ModeIsDTTransitionAvailable  ( void );
static BYTE X1ModeIsPSTransitionAvailable  ( void );
static BYTE X1ModeIsCmdTransitionAvailable ( void );*/

static BYTE X2ModeIsDTTransitionAvailable  ( void );
static BYTE X2ModeIsPSTransitionAvailable  ( void );
static BYTE X2ModeIsCmdTransitionAvailable ( void );

#endif